# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    # Package : Health Inpatient Calendar
#    Copyright (C) 2008-2017  Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2012  Sebastián Marró <smarro@thymbra.com>
#    Copyright (C) 2011-2017  GNU Solidario <health@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields
from trytond.pool import Pool


__all__ = ['HospitalBed', 'InpatientRegistration']


class HospitalBed(ModelSQL, ModelView):
    "Add Calendar to Hospital Bed"
    __name__ = "gnuhealth.hospital.bed"

    calendar = fields.Many2One('calendar.calendar', 'Calendar')


class InpatientRegistration(ModelSQL, ModelView):
    'Add Calendar to the Inpatient Registration'
    __name__ = 'gnuhealth.inpatient.registration'

    event = fields.Many2One('calendar.event', 'Calendar Event', readonly=True,
        help="Calendar Event")

    @classmethod
    def confirmed(cls, registrations):
        super(InpatientRegistration, cls).confirmed(registrations)

        Event = Pool().get('calendar.event')

        for inpatient_registration in registrations:
            if inpatient_registration.bed.calendar:
                if not inpatient_registration.event:
                    bed = inpatient_registration.bed.name.code + ": "
                    events = Event.create([{
                        'dtstart': inpatient_registration.hospitalization_date,
                        'dtend': inpatient_registration.discharge_date,
                        'calendar': inpatient_registration.bed.calendar.id,
                        'summary': bed + inpatient_registration.patient.name.rec_name
                        }])
                    cls.write([inpatient_registration],
                        {'event': events[0].id})

    @classmethod
    def discharge(cls, registrations):
        super(InpatientRegistration, cls).discharge(registrations)

        Event = Pool().get('calendar.event')

        for inpatient_registration in registrations:
            if inpatient_registration.event:
                Event.delete([inpatient_registration.event])

    @classmethod
    def write(cls, registrations, values):
        Event = Pool().get('calendar.event')
        Patient = Pool().get('gnuhealth.patient')
        HospitalBed = Pool().get('gnuhealth.hospital.bed')

        for inpatient_registration in registrations:
            if inpatient_registration.event:
                if 'hospitalization_date' in values:
                    Event.write([inpatient_registration.event], {
                        'dtstart': values['hospitalization_date'],
                        })
                if 'discharge_date' in values:
                    Event.write([inpatient_registration.event], {
                        'dtend': values['discharge_date'],
                        })
                if 'bed' in values:
                    bed = HospitalBed(values['bed'])
                    Event.write([inpatient_registration.event], {
                        'calendar': bed.calendar.id,
                        })
                if 'patient' in values:
                    patient = Patient(values['patient'])
                    bed = inpatient_registration.bed.name.code + ": "
                    Event.write([inpatient_registration.event], {
                        'summary': bed + patient.name.rec_name,
                        })

        return super(InpatientRegistration, cls).write(registrations, values)

    @classmethod
    def delete(cls, registrations):
        Event = Pool().get('calendar.event')

        for inpatient_registration in registrations:
            if inpatient_registration.event:
                Event.delete([inpatient_registration.event])
        return super(InpatientRegistration, cls).delete(registrations)



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import pytz
from datetime import datetime
from trytond.pool import Pool
from trytond.transaction import Transaction
from trytond.report import Report

__all__ = ['PatientDiseaseReport', 'PatientMedicationReport', 
    'PatientVaccinationReport']

def get_print_date():
    Company = Pool().get('company.company')

    timezone = None
    company_id = Transaction().context.get('company')
    if company_id:
        company = Company(company_id)
        if company.timezone:
            timezone = pytz.timezone(company.timezone)

    dt = datetime.now()
    return datetime.astimezone(dt.replace(tzinfo=pytz.utc), timezone)


class PatientDiseaseReport(Report):
    __name__ = 'patient.disease'

    @classmethod
    def parse(cls, report, objects, data, localcontext):
        localcontext['print_date'] = get_print_date()
        localcontext['print_time'] = localcontext['print_date'].time()

        return super(PatientDiseaseReport, cls).parse(report, objects, data, 
            localcontext)


class PatientMedicationReport(Report):
    __name__ = 'patient.medication'

    @classmethod
    def parse(cls, report, objects, data, localcontext):
        localcontext['print_date'] = get_print_date()
        localcontext['print_time'] = localcontext['print_date'].time()

        return super(PatientMedicationReport, cls).parse(report, objects, data, 
            localcontext)


class PatientVaccinationReport(Report):
    __name__ = 'patient.vaccination'

    @classmethod
    def parse(cls, report, objects, data, localcontext):
        localcontext['print_date'] = get_print_date()
        localcontext['print_time'] = localcontext['print_date'].time()

        return super(PatientVaccinationReport, cls).parse(report, objects, data, 
            localcontext)



from .base_classes import *
from .support_functions import *


class DocumentReference(Resource):
    """A reference to a document.If the element is present, it must have
    either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, masterIdentifier=None, identifier=None, subject=None, type_=None, class_=None, author=None, custodian=None, policyManager=None, authenticator=None, created=None, indexed=None, status=None, docStatus=None, relatesTo=None, description=None, confidentiality=None, primaryLanguage=None, mimeType=None, format=None, size=None, hash=None, location=None, service=None, context=None):
        self.original_tagname_ = None
        super(DocumentReference, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.masterIdentifier = masterIdentifier
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.subject = subject
        self.type_ = type_
        self.class_ = class_
        if author is None:
            self.author = []
        else:
            self.author = author
        self.custodian = custodian
        self.policyManager = policyManager
        self.authenticator = authenticator
        self.created = created
        self.indexed = indexed
        self.status = status
        self.docStatus = docStatus
        if relatesTo is None:
            self.relatesTo = []
        else:
            self.relatesTo = relatesTo
        self.description = description
        if confidentiality is None:
            self.confidentiality = []
        else:
            self.confidentiality = confidentiality
        self.primaryLanguage = primaryLanguage
        self.mimeType = mimeType
        if format is None:
            self.format = []
        else:
            self.format = format
        self.size = size
        self.hash = hash
        self.location = location
        self.service = service
        self.context = context
    def factory(*args_, **kwargs_):
        if DocumentReference.subclass:
            return DocumentReference.subclass(*args_, **kwargs_)
        else:
            return DocumentReference(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_masterIdentifier(self): return self.masterIdentifier
    def set_masterIdentifier(self, masterIdentifier): self.masterIdentifier = masterIdentifier
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_class(self): return self.class_
    def set_class(self, class_): self.class_ = class_
    def get_author(self): return self.author
    def set_author(self, author): self.author = author
    def add_author(self, value): self.author.append(value)
    def insert_author(self, index, value): self.author[index] = value
    def get_custodian(self): return self.custodian
    def set_custodian(self, custodian): self.custodian = custodian
    def get_policyManager(self): return self.policyManager
    def set_policyManager(self, policyManager): self.policyManager = policyManager
    def get_authenticator(self): return self.authenticator
    def set_authenticator(self, authenticator): self.authenticator = authenticator
    def get_created(self): return self.created
    def set_created(self, created): self.created = created
    def get_indexed(self): return self.indexed
    def set_indexed(self, indexed): self.indexed = indexed
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_docStatus(self): return self.docStatus
    def set_docStatus(self, docStatus): self.docStatus = docStatus
    def get_relatesTo(self): return self.relatesTo
    def set_relatesTo(self, relatesTo): self.relatesTo = relatesTo
    def add_relatesTo(self, value): self.relatesTo.append(value)
    def insert_relatesTo(self, index, value): self.relatesTo[index] = value
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_confidentiality(self): return self.confidentiality
    def set_confidentiality(self, confidentiality): self.confidentiality = confidentiality
    def add_confidentiality(self, value): self.confidentiality.append(value)
    def insert_confidentiality(self, index, value): self.confidentiality[index] = value
    def get_primaryLanguage(self): return self.primaryLanguage
    def set_primaryLanguage(self, primaryLanguage): self.primaryLanguage = primaryLanguage
    def get_mimeType(self): return self.mimeType
    def set_mimeType(self, mimeType): self.mimeType = mimeType
    def get_format(self): return self.format
    def set_format(self, format): self.format = format
    def add_format(self, value): self.format.append(value)
    def insert_format(self, index, value): self.format[index] = value
    def get_size(self): return self.size
    def set_size(self, size): self.size = size
    def get_hash(self): return self.hash
    def set_hash(self, hash): self.hash = hash
    def get_location(self): return self.location
    def set_location(self, location): self.location = location
    def get_service(self): return self.service
    def set_service(self, service): self.service = service
    def get_context(self): return self.context
    def set_context(self, context): self.context = context
    def hasContent_(self):
        if (
            self.masterIdentifier is not None or
            self.identifier or
            self.subject is not None or
            self.type_ is not None or
            self.class_ is not None or
            self.author or
            self.custodian is not None or
            self.policyManager is not None or
            self.authenticator is not None or
            self.created is not None or
            self.indexed is not None or
            self.status is not None or
            self.docStatus is not None or
            self.relatesTo or
            self.description is not None or
            self.confidentiality or
            self.primaryLanguage is not None or
            self.mimeType is not None or
            self.format or
            self.size is not None or
            self.hash is not None or
            self.location is not None or
            self.service is not None or
            self.context is not None or
            super(DocumentReference, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentReference', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentReference', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentReference'):
        super(DocumentReference, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference')
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentReference', fromsubclass_=False, pretty_print=True):
        super(DocumentReference, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.masterIdentifier is not None:
            self.masterIdentifier.export(outfile, level, namespace_, name_='masterIdentifier', pretty_print=pretty_print)
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.class_ is not None:
            self.class_.export(outfile, level, namespace_, name_='class', pretty_print=pretty_print)
        for author_ in self.author:
            author_.export(outfile, level, namespace_, name_='author', pretty_print=pretty_print)
        if self.custodian is not None:
            self.custodian.export(outfile, level, namespace_, name_='custodian', pretty_print=pretty_print)
        if self.policyManager is not None:
            self.policyManager.export(outfile, level, namespace_, name_='policyManager', pretty_print=pretty_print)
        if self.authenticator is not None:
            self.authenticator.export(outfile, level, namespace_, name_='authenticator', pretty_print=pretty_print)
        if self.created is not None:
            self.created.export(outfile, level, namespace_, name_='created', pretty_print=pretty_print)
        if self.indexed is not None:
            self.indexed.export(outfile, level, namespace_, name_='indexed', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.docStatus is not None:
            self.docStatus.export(outfile, level, namespace_, name_='docStatus', pretty_print=pretty_print)
        for relatesTo_ in self.relatesTo:
            relatesTo_.export(outfile, level, namespace_, name_='relatesTo', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        for confidentiality_ in self.confidentiality:
            confidentiality_.export(outfile, level, namespace_, name_='confidentiality', pretty_print=pretty_print)
        if self.primaryLanguage is not None:
            self.primaryLanguage.export(outfile, level, namespace_, name_='primaryLanguage', pretty_print=pretty_print)
        if self.mimeType is not None:
            self.mimeType.export(outfile, level, namespace_, name_='mimeType', pretty_print=pretty_print)
        for format_ in self.format:
            format_.export(outfile, level, namespace_, name_='format', pretty_print=pretty_print)
        if self.size is not None:
            self.size.export(outfile, level, namespace_, name_='size', pretty_print=pretty_print)
        if self.hash is not None:
            self.hash.export(outfile, level, namespace_, name_='hash', pretty_print=pretty_print)
        if self.location is not None:
            self.location.export(outfile, level, namespace_, name_='location', pretty_print=pretty_print)
        if self.service is not None:
            self.service.export(outfile, level, namespace_, name_='service', pretty_print=pretty_print)
        if self.context is not None:
            self.context.export(outfile, level, namespace_, name_='context', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentReference', mapping_=None):
        element = super(DocumentReference, self).to_etree(parent_element, name_, mapping_)
        if self.masterIdentifier is not None:
            masterIdentifier_ = self.masterIdentifier
            masterIdentifier_.to_etree(element, name_='masterIdentifier', mapping_=mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.class_ is not None:
            class__ = self.class_
            class__.to_etree(element, name_='class', mapping_=mapping_)
        for author_ in self.author:
            author_.to_etree(element, name_='author', mapping_=mapping_)
        if self.custodian is not None:
            custodian_ = self.custodian
            custodian_.to_etree(element, name_='custodian', mapping_=mapping_)
        if self.policyManager is not None:
            policyManager_ = self.policyManager
            policyManager_.to_etree(element, name_='policyManager', mapping_=mapping_)
        if self.authenticator is not None:
            authenticator_ = self.authenticator
            authenticator_.to_etree(element, name_='authenticator', mapping_=mapping_)
        if self.created is not None:
            created_ = self.created
            created_.to_etree(element, name_='created', mapping_=mapping_)
        if self.indexed is not None:
            indexed_ = self.indexed
            indexed_.to_etree(element, name_='indexed', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.docStatus is not None:
            docStatus_ = self.docStatus
            docStatus_.to_etree(element, name_='docStatus', mapping_=mapping_)
        for relatesTo_ in self.relatesTo:
            relatesTo_.to_etree(element, name_='relatesTo', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        for confidentiality_ in self.confidentiality:
            confidentiality_.to_etree(element, name_='confidentiality', mapping_=mapping_)
        if self.primaryLanguage is not None:
            primaryLanguage_ = self.primaryLanguage
            primaryLanguage_.to_etree(element, name_='primaryLanguage', mapping_=mapping_)
        if self.mimeType is not None:
            mimeType_ = self.mimeType
            mimeType_.to_etree(element, name_='mimeType', mapping_=mapping_)
        for format_ in self.format:
            format_.to_etree(element, name_='format', mapping_=mapping_)
        if self.size is not None:
            size_ = self.size
            size_.to_etree(element, name_='size', mapping_=mapping_)
        if self.hash is not None:
            hash_ = self.hash
            hash_.to_etree(element, name_='hash', mapping_=mapping_)
        if self.location is not None:
            location_ = self.location
            location_.to_etree(element, name_='location', mapping_=mapping_)
        if self.service is not None:
            service_ = self.service
            service_.to_etree(element, name_='service', mapping_=mapping_)
        if self.context is not None:
            context_ = self.context
            context_.to_etree(element, name_='context', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DocumentReference, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'masterIdentifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.masterIdentifier = obj_
            obj_.original_tagname_ = 'masterIdentifier'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'class':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.class_ = obj_
            obj_.original_tagname_ = 'class'
        elif nodeName_ == 'author':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.author.append(obj_)
            obj_.original_tagname_ = 'author'
        elif nodeName_ == 'custodian':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.custodian = obj_
            obj_.original_tagname_ = 'custodian'
        elif nodeName_ == 'policyManager':
            obj_ = uri.factory()
            obj_.build(child_)
            self.policyManager = obj_
            obj_.original_tagname_ = 'policyManager'
        elif nodeName_ == 'authenticator':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.authenticator = obj_
            obj_.original_tagname_ = 'authenticator'
        elif nodeName_ == 'created':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.created = obj_
            obj_.original_tagname_ = 'created'
        elif nodeName_ == 'indexed':
            obj_ = instant.factory()
            obj_.build(child_)
            self.indexed = obj_
            obj_.original_tagname_ = 'indexed'
        elif nodeName_ == 'status':
            obj_ = DocumentReferenceStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'docStatus':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.docStatus = obj_
            obj_.original_tagname_ = 'docStatus'
        elif nodeName_ == 'relatesTo':
            obj_ = DocumentReference_RelatesTo.factory()
            obj_.build(child_)
            self.relatesTo.append(obj_)
            obj_.original_tagname_ = 'relatesTo'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'confidentiality':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.confidentiality.append(obj_)
            obj_.original_tagname_ = 'confidentiality'
        elif nodeName_ == 'primaryLanguage':
            obj_ = code.factory()
            obj_.build(child_)
            self.primaryLanguage = obj_
            obj_.original_tagname_ = 'primaryLanguage'
        elif nodeName_ == 'mimeType':
            obj_ = code.factory()
            obj_.build(child_)
            self.mimeType = obj_
            obj_.original_tagname_ = 'mimeType'
        elif nodeName_ == 'format':
            obj_ = uri.factory()
            obj_.build(child_)
            self.format.append(obj_)
            obj_.original_tagname_ = 'format'
        elif nodeName_ == 'size':
            obj_ = integer.factory()
            obj_.build(child_)
            self.size = obj_
            obj_.original_tagname_ = 'size'
        elif nodeName_ == 'hash':
            obj_ = string.factory()
            obj_.build(child_)
            self.hash = obj_
            obj_.original_tagname_ = 'hash'
        elif nodeName_ == 'location':
            obj_ = uri.factory()
            obj_.build(child_)
            self.location = obj_
            obj_.original_tagname_ = 'location'
        elif nodeName_ == 'service':
            obj_ = DocumentReference_Service.factory()
            obj_.build(child_)
            self.service = obj_
            obj_.original_tagname_ = 'service'
        elif nodeName_ == 'context':
            obj_ = DocumentReference_Context.factory()
            obj_.build(child_)
            self.context = obj_
            obj_.original_tagname_ = 'context'
        super(DocumentReference, self).buildChildren(child_, node, nodeName_, True)
# end class DocumentReference


class DocumentReference_RelatesTo(BackboneElement):
    """A reference to a document."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, target=None):
        self.original_tagname_ = None
        super(DocumentReference_RelatesTo, self).__init__(id, extension, modifierExtension, )
        self.code = code
        self.target = target
    def factory(*args_, **kwargs_):
        if DocumentReference_RelatesTo.subclass:
            return DocumentReference_RelatesTo.subclass(*args_, **kwargs_)
        else:
            return DocumentReference_RelatesTo(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_target(self): return self.target
    def set_target(self, target): self.target = target
    def hasContent_(self):
        if (
            self.code is not None or
            self.target is not None or
            super(DocumentReference_RelatesTo, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentReference.RelatesTo', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.RelatesTo')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentReference.RelatesTo', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentReference.RelatesTo'):
        super(DocumentReference_RelatesTo, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.RelatesTo')
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentReference.RelatesTo', fromsubclass_=False, pretty_print=True):
        super(DocumentReference_RelatesTo, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.target is not None:
            self.target.export(outfile, level, namespace_, name_='target', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentReference.RelatesTo', mapping_=None):
        element = super(DocumentReference_RelatesTo, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.target is not None:
            target_ = self.target
            target_.to_etree(element, name_='target', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DocumentReference_RelatesTo, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = DocumentRelationshipType.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'target':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.target = obj_
            obj_.original_tagname_ = 'target'
        super(DocumentReference_RelatesTo, self).buildChildren(child_, node, nodeName_, True)
# end class DocumentReference_RelatesTo


class DocumentReference_Service(BackboneElement):
    """A reference to a document."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, type_=None, address=None, parameter=None):
        self.original_tagname_ = None
        super(DocumentReference_Service, self).__init__(id, extension, modifierExtension, )
        self.type_ = type_
        self.address = address
        if parameter is None:
            self.parameter = []
        else:
            self.parameter = parameter
    def factory(*args_, **kwargs_):
        if DocumentReference_Service.subclass:
            return DocumentReference_Service.subclass(*args_, **kwargs_)
        else:
            return DocumentReference_Service(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_address(self): return self.address
    def set_address(self, address): self.address = address
    def get_parameter(self): return self.parameter
    def set_parameter(self, parameter): self.parameter = parameter
    def add_parameter(self, value): self.parameter.append(value)
    def insert_parameter(self, index, value): self.parameter[index] = value
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.address is not None or
            self.parameter or
            super(DocumentReference_Service, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentReference.Service', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.Service')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentReference.Service', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentReference.Service'):
        super(DocumentReference_Service, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.Service')
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentReference.Service', fromsubclass_=False, pretty_print=True):
        super(DocumentReference_Service, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.address is not None:
            self.address.export(outfile, level, namespace_, name_='address', pretty_print=pretty_print)
        for parameter_ in self.parameter:
            parameter_.export(outfile, level, namespace_, name_='parameter', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentReference.Service', mapping_=None):
        element = super(DocumentReference_Service, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.address is not None:
            address_ = self.address
            address_.to_etree(element, name_='address', mapping_=mapping_)
        for parameter_ in self.parameter:
            parameter_.to_etree(element, name_='parameter', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DocumentReference_Service, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'address':
            obj_ = string.factory()
            obj_.build(child_)
            self.address = obj_
            obj_.original_tagname_ = 'address'
        elif nodeName_ == 'parameter':
            obj_ = DocumentReference_Parameter.factory()
            obj_.build(child_)
            self.parameter.append(obj_)
            obj_.original_tagname_ = 'parameter'
        super(DocumentReference_Service, self).buildChildren(child_, node, nodeName_, True)
# end class DocumentReference_Service


class DocumentReference_Parameter(BackboneElement):
    """A reference to a document."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, name=None, value=None):
        self.original_tagname_ = None
        super(DocumentReference_Parameter, self).__init__(id, extension, modifierExtension, )
        self.name = name
        self.value = value
    def factory(*args_, **kwargs_):
        if DocumentReference_Parameter.subclass:
            return DocumentReference_Parameter.subclass(*args_, **kwargs_)
        else:
            return DocumentReference_Parameter(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            self.name is not None or
            self.value is not None or
            super(DocumentReference_Parameter, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentReference.Parameter', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.Parameter')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentReference.Parameter', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentReference.Parameter'):
        super(DocumentReference_Parameter, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.Parameter')
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentReference.Parameter', fromsubclass_=False, pretty_print=True):
        super(DocumentReference_Parameter, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.value is not None:
            self.value.export(outfile, level, namespace_, name_='value', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentReference.Parameter', mapping_=None):
        element = super(DocumentReference_Parameter, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.value is not None:
            value_ = self.value
            value_.to_etree(element, name_='value', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DocumentReference_Parameter, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = string.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'value':
            obj_ = string.factory()
            obj_.build(child_)
            self.value = obj_
            obj_.original_tagname_ = 'value'
        super(DocumentReference_Parameter, self).buildChildren(child_, node, nodeName_, True)
# end class DocumentReference_Parameter


class DocumentReference_Context(BackboneElement):
    """A reference to a document."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, event=None, period=None, facilityType=None):
        self.original_tagname_ = None
        super(DocumentReference_Context, self).__init__(id, extension, modifierExtension, )
        if event is None:
            self.event = []
        else:
            self.event = event
        self.period = period
        self.facilityType = facilityType
    def factory(*args_, **kwargs_):
        if DocumentReference_Context.subclass:
            return DocumentReference_Context.subclass(*args_, **kwargs_)
        else:
            return DocumentReference_Context(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_event(self): return self.event
    def set_event(self, event): self.event = event
    def add_event(self, value): self.event.append(value)
    def insert_event(self, index, value): self.event[index] = value
    def get_period(self): return self.period
    def set_period(self, period): self.period = period
    def get_facilityType(self): return self.facilityType
    def set_facilityType(self, facilityType): self.facilityType = facilityType
    def hasContent_(self):
        if (
            self.event or
            self.period is not None or
            self.facilityType is not None or
            super(DocumentReference_Context, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentReference.Context', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.Context')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentReference.Context', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentReference.Context'):
        super(DocumentReference_Context, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReference.Context')
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentReference.Context', fromsubclass_=False, pretty_print=True):
        super(DocumentReference_Context, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for event_ in self.event:
            event_.export(outfile, level, namespace_, name_='event', pretty_print=pretty_print)
        if self.period is not None:
            self.period.export(outfile, level, namespace_, name_='period', pretty_print=pretty_print)
        if self.facilityType is not None:
            self.facilityType.export(outfile, level, namespace_, name_='facilityType', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentReference.Context', mapping_=None):
        element = super(DocumentReference_Context, self).to_etree(parent_element, name_, mapping_)
        for event_ in self.event:
            event_.to_etree(element, name_='event', mapping_=mapping_)
        if self.period is not None:
            period_ = self.period
            period_.to_etree(element, name_='period', mapping_=mapping_)
        if self.facilityType is not None:
            facilityType_ = self.facilityType
            facilityType_.to_etree(element, name_='facilityType', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DocumentReference_Context, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'event':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.event.append(obj_)
            obj_.original_tagname_ = 'event'
        elif nodeName_ == 'period':
            obj_ = Period.factory()
            obj_.build(child_)
            self.period = obj_
            obj_.original_tagname_ = 'period'
        elif nodeName_ == 'facilityType':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.facilityType = obj_
            obj_.original_tagname_ = 'facilityType'
        super(DocumentReference_Context, self).buildChildren(child_, node, nodeName_, True)
# end class DocumentReference_Context


class DocumentReferenceStatus(Element):
    """If the element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(DocumentReferenceStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if DocumentReferenceStatus.subclass:
            return DocumentReferenceStatus.subclass(*args_, **kwargs_)
        else:
            return DocumentReferenceStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(DocumentReferenceStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentReferenceStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReferenceStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentReferenceStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentReferenceStatus'):
        super(DocumentReferenceStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentReferenceStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentReferenceStatus', fromsubclass_=False, pretty_print=True):
        super(DocumentReferenceStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentReferenceStatus', mapping_=None):
        element = super(DocumentReferenceStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(DocumentReferenceStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(DocumentReferenceStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class DocumentReferenceStatus



from .base_classes import *
from .support_functions import *


class Binary(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, contentType=None, id=None, valueOf_=None):
        self.original_tagname_ = None
        self.contentType = _cast(None, contentType)
        self.id = _cast(None, id)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if Binary.subclass:
            return Binary.subclass(*args_, **kwargs_)
        else:
            return Binary(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_contentType(self): return self.contentType
    def set_contentType(self, contentType): self.contentType = contentType
    def get_id(self): return self.id
    def set_id(self, id): self.id = id
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Binary', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Binary')
        if self.hasContent_():
            outfile.write('>')
            outfile.write(str(self.valueOf_).encode(ExternalEncoding))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Binary', pretty_print=pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Binary'):
        if self.contentType is not None and 'contentType' not in already_processed:
            already_processed.add('contentType')
            outfile.write(' contentType=%s' % (self.gds_format_string(quote_attrib(self.contentType).encode(ExternalEncoding), input_name='contentType'), ))
        if self.id is not None and 'id' not in already_processed:
            already_processed.add('id')
            outfile.write(' id=%s' % (quote_attrib(self.id), ))
    def exportChildren(self, outfile, level, namespace_='', name_='Binary', fromsubclass_=False, pretty_print=True):
        pass
    def to_etree(self, parent_element=None, name_='Binary', mapping_=None):
        if parent_element is None:
            element = etree_.Element('{http://hl7.org/fhir}' + name_)
        else:
            element = etree_.SubElement(parent_element, '{http://hl7.org/fhir}' + name_)
        if self.contentType is not None:
            element.set('contentType', self.gds_format_string(self.contentType))
        if self.id is not None:
            element.set('id', self.id)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('contentType', node)
        if value is not None and 'contentType' not in already_processed:
            already_processed.add('contentType')
            self.contentType = value
        value = find_attr_value_('id', node)
        if value is not None and 'id' not in already_processed:
            already_processed.add('id')
            self.id = value
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class Binary



from .base_classes import *
from .support_functions import *


class Substance(Resource):
    """A homogeneous material with a definite composition.If the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, type_=None, description=None, instance=None, ingredient=None):
        self.original_tagname_ = None
        super(Substance, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.type_ = type_
        self.description = description
        self.instance = instance
        if ingredient is None:
            self.ingredient = []
        else:
            self.ingredient = ingredient
    def factory(*args_, **kwargs_):
        if Substance.subclass:
            return Substance.subclass(*args_, **kwargs_)
        else:
            return Substance(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_instance(self): return self.instance
    def set_instance(self, instance): self.instance = instance
    def get_ingredient(self): return self.ingredient
    def set_ingredient(self, ingredient): self.ingredient = ingredient
    def add_ingredient(self, value): self.ingredient.append(value)
    def insert_ingredient(self, index, value): self.ingredient[index] = value
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.description is not None or
            self.instance is not None or
            self.ingredient or
            super(Substance, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Substance', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Substance')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Substance', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Substance'):
        super(Substance, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Substance')
    def exportChildren(self, outfile, level, namespace_='', name_='Substance', fromsubclass_=False, pretty_print=True):
        super(Substance, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.instance is not None:
            self.instance.export(outfile, level, namespace_, name_='instance', pretty_print=pretty_print)
        for ingredient_ in self.ingredient:
            ingredient_.export(outfile, level, namespace_, name_='ingredient', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Substance', mapping_=None):
        element = super(Substance, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.instance is not None:
            instance_ = self.instance
            instance_.to_etree(element, name_='instance', mapping_=mapping_)
        for ingredient_ in self.ingredient:
            ingredient_.to_etree(element, name_='ingredient', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Substance, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'instance':
            obj_ = Substance_Instance.factory()
            obj_.build(child_)
            self.instance = obj_
            obj_.original_tagname_ = 'instance'
        elif nodeName_ == 'ingredient':
            obj_ = Substance_Ingredient.factory()
            obj_.build(child_)
            self.ingredient.append(obj_)
            obj_.original_tagname_ = 'ingredient'
        super(Substance, self).buildChildren(child_, node, nodeName_, True)
# end class Substance


class Substance_Instance(BackboneElement):
    """A homogeneous material with a definite composition."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, identifier=None, expiry=None, quantity=None):
        self.original_tagname_ = None
        super(Substance_Instance, self).__init__(id, extension, modifierExtension, )
        self.identifier = identifier
        self.expiry = expiry
        self.quantity = quantity
    def factory(*args_, **kwargs_):
        if Substance_Instance.subclass:
            return Substance_Instance.subclass(*args_, **kwargs_)
        else:
            return Substance_Instance(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_expiry(self): return self.expiry
    def set_expiry(self, expiry): self.expiry = expiry
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.expiry is not None or
            self.quantity is not None or
            super(Substance_Instance, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Substance.Instance', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Substance.Instance')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Substance.Instance', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Substance.Instance'):
        super(Substance_Instance, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Substance.Instance')
    def exportChildren(self, outfile, level, namespace_='', name_='Substance.Instance', fromsubclass_=False, pretty_print=True):
        super(Substance_Instance, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.expiry is not None:
            self.expiry.export(outfile, level, namespace_, name_='expiry', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Substance.Instance', mapping_=None):
        element = super(Substance_Instance, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.expiry is not None:
            expiry_ = self.expiry
            expiry_.to_etree(element, name_='expiry', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Substance_Instance, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'expiry':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.expiry = obj_
            obj_.original_tagname_ = 'expiry'
        elif nodeName_ == 'quantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        super(Substance_Instance, self).buildChildren(child_, node, nodeName_, True)
# end class Substance_Instance


class Substance_Ingredient(BackboneElement):
    """A homogeneous material with a definite composition."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, quantity=None, substance=None):
        self.original_tagname_ = None
        super(Substance_Ingredient, self).__init__(id, extension, modifierExtension, )
        self.quantity = quantity
        self.substance = substance
    def factory(*args_, **kwargs_):
        if Substance_Ingredient.subclass:
            return Substance_Ingredient.subclass(*args_, **kwargs_)
        else:
            return Substance_Ingredient(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_substance(self): return self.substance
    def set_substance(self, substance): self.substance = substance
    def hasContent_(self):
        if (
            self.quantity is not None or
            self.substance is not None or
            super(Substance_Ingredient, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Substance.Ingredient', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Substance.Ingredient')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Substance.Ingredient', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Substance.Ingredient'):
        super(Substance_Ingredient, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Substance.Ingredient')
    def exportChildren(self, outfile, level, namespace_='', name_='Substance.Ingredient', fromsubclass_=False, pretty_print=True):
        super(Substance_Ingredient, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        if self.substance is not None:
            self.substance.export(outfile, level, namespace_, name_='substance', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Substance.Ingredient', mapping_=None):
        element = super(Substance_Ingredient, self).to_etree(parent_element, name_, mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if self.substance is not None:
            substance_ = self.substance
            substance_.to_etree(element, name_='substance', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Substance_Ingredient, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'quantity':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'substance':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.substance = obj_
            obj_.original_tagname_ = 'substance'
        super(Substance_Ingredient, self).buildChildren(child_, node, nodeName_, True)
# end class Substance_Ingredient



from .base_classes import *
from .support_functions import *


class ValueSet(Resource):
    """A value set specifies a set of codes drawn from one or more code
    systems.If the element is present, it must have either a @value,
    an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, version=None, name=None, publisher=None, telecom=None, description=None, copyright=None, status=None, experimental=None, extensible=None, date=None, define=None, compose=None, expansion=None):
        self.original_tagname_ = None
        super(ValueSet, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.identifier = identifier
        self.version = version
        self.name = name
        self.publisher = publisher
        if telecom is None:
            self.telecom = []
        else:
            self.telecom = telecom
        self.description = description
        self.copyright = copyright
        self.status = status
        self.experimental = experimental
        self.extensible = extensible
        self.date = date
        self.define = define
        self.compose = compose
        self.expansion = expansion
    def factory(*args_, **kwargs_):
        if ValueSet.subclass:
            return ValueSet.subclass(*args_, **kwargs_)
        else:
            return ValueSet(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_version(self): return self.version
    def set_version(self, version): self.version = version
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_publisher(self): return self.publisher
    def set_publisher(self, publisher): self.publisher = publisher
    def get_telecom(self): return self.telecom
    def set_telecom(self, telecom): self.telecom = telecom
    def add_telecom(self, value): self.telecom.append(value)
    def insert_telecom(self, index, value): self.telecom[index] = value
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_copyright(self): return self.copyright
    def set_copyright(self, copyright): self.copyright = copyright
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_experimental(self): return self.experimental
    def set_experimental(self, experimental): self.experimental = experimental
    def get_extensible(self): return self.extensible
    def set_extensible(self, extensible): self.extensible = extensible
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_define(self): return self.define
    def set_define(self, define): self.define = define
    def get_compose(self): return self.compose
    def set_compose(self, compose): self.compose = compose
    def get_expansion(self): return self.expansion
    def set_expansion(self, expansion): self.expansion = expansion
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.version is not None or
            self.name is not None or
            self.publisher is not None or
            self.telecom or
            self.description is not None or
            self.copyright is not None or
            self.status is not None or
            self.experimental is not None or
            self.extensible is not None or
            self.date is not None or
            self.define is not None or
            self.compose is not None or
            self.expansion is not None or
            super(ValueSet, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet'):
        super(ValueSet, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet', fromsubclass_=False, pretty_print=True):
        super(ValueSet, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.version is not None:
            self.version.export(outfile, level, namespace_, name_='version', pretty_print=pretty_print)
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.publisher is not None:
            self.publisher.export(outfile, level, namespace_, name_='publisher', pretty_print=pretty_print)
        for telecom_ in self.telecom:
            telecom_.export(outfile, level, namespace_, name_='telecom', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.copyright is not None:
            self.copyright.export(outfile, level, namespace_, name_='copyright', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.experimental is not None:
            self.experimental.export(outfile, level, namespace_, name_='experimental', pretty_print=pretty_print)
        if self.extensible is not None:
            self.extensible.export(outfile, level, namespace_, name_='extensible', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.define is not None:
            self.define.export(outfile, level, namespace_, name_='define', pretty_print=pretty_print)
        if self.compose is not None:
            self.compose.export(outfile, level, namespace_, name_='compose', pretty_print=pretty_print)
        if self.expansion is not None:
            self.expansion.export(outfile, level, namespace_, name_='expansion', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet', mapping_=None):
        element = super(ValueSet, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.version is not None:
            version_ = self.version
            version_.to_etree(element, name_='version', mapping_=mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.publisher is not None:
            publisher_ = self.publisher
            publisher_.to_etree(element, name_='publisher', mapping_=mapping_)
        for telecom_ in self.telecom:
            telecom_.to_etree(element, name_='telecom', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.copyright is not None:
            copyright_ = self.copyright
            copyright_.to_etree(element, name_='copyright', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.experimental is not None:
            experimental_ = self.experimental
            experimental_.to_etree(element, name_='experimental', mapping_=mapping_)
        if self.extensible is not None:
            extensible_ = self.extensible
            extensible_.to_etree(element, name_='extensible', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.define is not None:
            define_ = self.define
            define_.to_etree(element, name_='define', mapping_=mapping_)
        if self.compose is not None:
            compose_ = self.compose
            compose_.to_etree(element, name_='compose', mapping_=mapping_)
        if self.expansion is not None:
            expansion_ = self.expansion
            expansion_.to_etree(element, name_='expansion', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = string.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'version':
            obj_ = string.factory()
            obj_.build(child_)
            self.version = obj_
            obj_.original_tagname_ = 'version'
        elif nodeName_ == 'name':
            obj_ = string.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'publisher':
            obj_ = string.factory()
            obj_.build(child_)
            self.publisher = obj_
            obj_.original_tagname_ = 'publisher'
        elif nodeName_ == 'telecom':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.telecom.append(obj_)
            obj_.original_tagname_ = 'telecom'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'copyright':
            obj_ = string.factory()
            obj_.build(child_)
            self.copyright = obj_
            obj_.original_tagname_ = 'copyright'
        elif nodeName_ == 'status':
            obj_ = ValueSetStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'experimental':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.experimental = obj_
            obj_.original_tagname_ = 'experimental'
        elif nodeName_ == 'extensible':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.extensible = obj_
            obj_.original_tagname_ = 'extensible'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'define':
            obj_ = ValueSet_Define.factory()
            obj_.build(child_)
            self.define = obj_
            obj_.original_tagname_ = 'define'
        elif nodeName_ == 'compose':
            obj_ = ValueSet_Compose.factory()
            obj_.build(child_)
            self.compose = obj_
            obj_.original_tagname_ = 'compose'
        elif nodeName_ == 'expansion':
            obj_ = ValueSet_Expansion.factory()
            obj_.build(child_)
            self.expansion = obj_
            obj_.original_tagname_ = 'expansion'
        super(ValueSet, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet


class ValueSet_Define(BackboneElement):
    """A value set specifies a set of codes drawn from one or more code
    systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, system=None, version=None, caseSensitive=None, concept=None):
        self.original_tagname_ = None
        super(ValueSet_Define, self).__init__(id, extension, modifierExtension, )
        self.system = system
        self.version = version
        self.caseSensitive = caseSensitive
        if concept is None:
            self.concept = []
        else:
            self.concept = concept
    def factory(*args_, **kwargs_):
        if ValueSet_Define.subclass:
            return ValueSet_Define.subclass(*args_, **kwargs_)
        else:
            return ValueSet_Define(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_system(self): return self.system
    def set_system(self, system): self.system = system
    def get_version(self): return self.version
    def set_version(self, version): self.version = version
    def get_caseSensitive(self): return self.caseSensitive
    def set_caseSensitive(self, caseSensitive): self.caseSensitive = caseSensitive
    def get_concept(self): return self.concept
    def set_concept(self, concept): self.concept = concept
    def add_concept(self, value): self.concept.append(value)
    def insert_concept(self, index, value): self.concept[index] = value
    def hasContent_(self):
        if (
            self.system is not None or
            self.version is not None or
            self.caseSensitive is not None or
            self.concept or
            super(ValueSet_Define, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet.Define', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Define')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet.Define', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet.Define'):
        super(ValueSet_Define, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Define')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet.Define', fromsubclass_=False, pretty_print=True):
        super(ValueSet_Define, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.system is not None:
            self.system.export(outfile, level, namespace_, name_='system', pretty_print=pretty_print)
        if self.version is not None:
            self.version.export(outfile, level, namespace_, name_='version', pretty_print=pretty_print)
        if self.caseSensitive is not None:
            self.caseSensitive.export(outfile, level, namespace_, name_='caseSensitive', pretty_print=pretty_print)
        for concept_ in self.concept:
            concept_.export(outfile, level, namespace_, name_='concept', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet.Define', mapping_=None):
        element = super(ValueSet_Define, self).to_etree(parent_element, name_, mapping_)
        if self.system is not None:
            system_ = self.system
            system_.to_etree(element, name_='system', mapping_=mapping_)
        if self.version is not None:
            version_ = self.version
            version_.to_etree(element, name_='version', mapping_=mapping_)
        if self.caseSensitive is not None:
            caseSensitive_ = self.caseSensitive
            caseSensitive_.to_etree(element, name_='caseSensitive', mapping_=mapping_)
        for concept_ in self.concept:
            concept_.to_etree(element, name_='concept', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet_Define, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'system':
            obj_ = uri.factory()
            obj_.build(child_)
            self.system = obj_
            obj_.original_tagname_ = 'system'
        elif nodeName_ == 'version':
            obj_ = string.factory()
            obj_.build(child_)
            self.version = obj_
            obj_.original_tagname_ = 'version'
        elif nodeName_ == 'caseSensitive':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.caseSensitive = obj_
            obj_.original_tagname_ = 'caseSensitive'
        elif nodeName_ == 'concept':
            obj_ = ValueSet_Concept.factory()
            obj_.build(child_)
            self.concept.append(obj_)
            obj_.original_tagname_ = 'concept'
        super(ValueSet_Define, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet_Define


class ValueSet_Concept(BackboneElement):
    """A value set specifies a set of codes drawn from one or more code
    systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, abstract=None, display=None, definition=None, concept=None):
        self.original_tagname_ = None
        super(ValueSet_Concept, self).__init__(id, extension, modifierExtension, )
        self.code = code
        self.abstract = abstract
        self.display = display
        self.definition = definition
        if concept is None:
            self.concept = []
        else:
            self.concept = concept
    def factory(*args_, **kwargs_):
        if ValueSet_Concept.subclass:
            return ValueSet_Concept.subclass(*args_, **kwargs_)
        else:
            return ValueSet_Concept(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_abstract(self): return self.abstract
    def set_abstract(self, abstract): self.abstract = abstract
    def get_display(self): return self.display
    def set_display(self, display): self.display = display
    def get_definition(self): return self.definition
    def set_definition(self, definition): self.definition = definition
    def get_concept(self): return self.concept
    def set_concept(self, concept): self.concept = concept
    def add_concept(self, value): self.concept.append(value)
    def insert_concept(self, index, value): self.concept[index] = value
    def hasContent_(self):
        if (
            self.code is not None or
            self.abstract is not None or
            self.display is not None or
            self.definition is not None or
            self.concept or
            super(ValueSet_Concept, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet.Concept', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Concept')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet.Concept', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet.Concept'):
        super(ValueSet_Concept, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Concept')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet.Concept', fromsubclass_=False, pretty_print=True):
        super(ValueSet_Concept, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.abstract is not None:
            self.abstract.export(outfile, level, namespace_, name_='abstract', pretty_print=pretty_print)
        if self.display is not None:
            self.display.export(outfile, level, namespace_, name_='display', pretty_print=pretty_print)
        if self.definition is not None:
            self.definition.export(outfile, level, namespace_, name_='definition', pretty_print=pretty_print)
        for concept_ in self.concept:
            concept_.export(outfile, level, namespace_, name_='concept', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet.Concept', mapping_=None):
        element = super(ValueSet_Concept, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.abstract is not None:
            abstract_ = self.abstract
            abstract_.to_etree(element, name_='abstract', mapping_=mapping_)
        if self.display is not None:
            display_ = self.display
            display_.to_etree(element, name_='display', mapping_=mapping_)
        if self.definition is not None:
            definition_ = self.definition
            definition_.to_etree(element, name_='definition', mapping_=mapping_)
        for concept_ in self.concept:
            concept_.to_etree(element, name_='concept', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet_Concept, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = code.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'abstract':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.abstract = obj_
            obj_.original_tagname_ = 'abstract'
        elif nodeName_ == 'display':
            obj_ = string.factory()
            obj_.build(child_)
            self.display = obj_
            obj_.original_tagname_ = 'display'
        elif nodeName_ == 'definition':
            obj_ = string.factory()
            obj_.build(child_)
            self.definition = obj_
            obj_.original_tagname_ = 'definition'
        elif nodeName_ == 'concept':
            obj_ = ValueSet_Concept.factory()
            obj_.build(child_)
            self.concept.append(obj_)
            obj_.original_tagname_ = 'concept'
        super(ValueSet_Concept, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet_Concept


class ValueSet_Compose(BackboneElement):
    """A value set specifies a set of codes drawn from one or more code
    systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, import_=None, include=None, exclude=None):
        self.original_tagname_ = None
        super(ValueSet_Compose, self).__init__(id, extension, modifierExtension, )
        if import_ is None:
            self.import_ = []
        else:
            self.import_ = import_
        if include is None:
            self.include = []
        else:
            self.include = include
        if exclude is None:
            self.exclude = []
        else:
            self.exclude = exclude
    def factory(*args_, **kwargs_):
        if ValueSet_Compose.subclass:
            return ValueSet_Compose.subclass(*args_, **kwargs_)
        else:
            return ValueSet_Compose(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_import(self): return self.import_
    def set_import(self, import_): self.import_ = import_
    def add_import(self, value): self.import_.append(value)
    def insert_import(self, index, value): self.import_[index] = value
    def get_include(self): return self.include
    def set_include(self, include): self.include = include
    def add_include(self, value): self.include.append(value)
    def insert_include(self, index, value): self.include[index] = value
    def get_exclude(self): return self.exclude
    def set_exclude(self, exclude): self.exclude = exclude
    def add_exclude(self, value): self.exclude.append(value)
    def insert_exclude(self, index, value): self.exclude[index] = value
    def hasContent_(self):
        if (
            self.import_ or
            self.include or
            self.exclude or
            super(ValueSet_Compose, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet.Compose', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Compose')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet.Compose', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet.Compose'):
        super(ValueSet_Compose, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Compose')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet.Compose', fromsubclass_=False, pretty_print=True):
        super(ValueSet_Compose, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for import_ in self.import_:
            import_.export(outfile, level, namespace_, name_='import', pretty_print=pretty_print)
        for include_ in self.include:
            include_.export(outfile, level, namespace_, name_='include', pretty_print=pretty_print)
        for exclude_ in self.exclude:
            exclude_.export(outfile, level, namespace_, name_='exclude', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet.Compose', mapping_=None):
        element = super(ValueSet_Compose, self).to_etree(parent_element, name_, mapping_)
        for import__ in self.import_:
            import__.to_etree(element, name_='import', mapping_=mapping_)
        for include_ in self.include:
            include_.to_etree(element, name_='include', mapping_=mapping_)
        for exclude_ in self.exclude:
            exclude_.to_etree(element, name_='exclude', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet_Compose, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'import':
            obj_ = uri.factory()
            obj_.build(child_)
            self.import_.append(obj_)
            obj_.original_tagname_ = 'import'
        elif nodeName_ == 'include':
            obj_ = ValueSet_Include.factory()
            obj_.build(child_)
            self.include.append(obj_)
            obj_.original_tagname_ = 'include'
        elif nodeName_ == 'exclude':
            obj_ = ValueSet_Include.factory()
            obj_.build(child_)
            self.exclude.append(obj_)
            obj_.original_tagname_ = 'exclude'
        super(ValueSet_Compose, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet_Compose


class ValueSet_Include(BackboneElement):
    """A value set specifies a set of codes drawn from one or more code
    systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, system=None, version=None, code=None, filter=None):
        self.original_tagname_ = None
        super(ValueSet_Include, self).__init__(id, extension, modifierExtension, )
        self.system = system
        self.version = version
        if code is None:
            self.code = []
        else:
            self.code = code
        if filter is None:
            self.filter = []
        else:
            self.filter = filter
    def factory(*args_, **kwargs_):
        if ValueSet_Include.subclass:
            return ValueSet_Include.subclass(*args_, **kwargs_)
        else:
            return ValueSet_Include(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_system(self): return self.system
    def set_system(self, system): self.system = system
    def get_version(self): return self.version
    def set_version(self, version): self.version = version
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def add_code(self, value): self.code.append(value)
    def insert_code(self, index, value): self.code[index] = value
    def get_filter(self): return self.filter
    def set_filter(self, filter): self.filter = filter
    def add_filter(self, value): self.filter.append(value)
    def insert_filter(self, index, value): self.filter[index] = value
    def hasContent_(self):
        if (
            self.system is not None or
            self.version is not None or
            self.code or
            self.filter or
            super(ValueSet_Include, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet.Include', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Include')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet.Include', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet.Include'):
        super(ValueSet_Include, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Include')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet.Include', fromsubclass_=False, pretty_print=True):
        super(ValueSet_Include, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.system is not None:
            self.system.export(outfile, level, namespace_, name_='system', pretty_print=pretty_print)
        if self.version is not None:
            self.version.export(outfile, level, namespace_, name_='version', pretty_print=pretty_print)
        for code_ in self.code:
            code_.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        for filter_ in self.filter:
            filter_.export(outfile, level, namespace_, name_='filter', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet.Include', mapping_=None):
        element = super(ValueSet_Include, self).to_etree(parent_element, name_, mapping_)
        if self.system is not None:
            system_ = self.system
            system_.to_etree(element, name_='system', mapping_=mapping_)
        if self.version is not None:
            version_ = self.version
            version_.to_etree(element, name_='version', mapping_=mapping_)
        for code_ in self.code:
            code_.to_etree(element, name_='code', mapping_=mapping_)
        for filter_ in self.filter:
            filter_.to_etree(element, name_='filter', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet_Include, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'system':
            obj_ = uri.factory()
            obj_.build(child_)
            self.system = obj_
            obj_.original_tagname_ = 'system'
        elif nodeName_ == 'version':
            obj_ = string.factory()
            obj_.build(child_)
            self.version = obj_
            obj_.original_tagname_ = 'version'
        elif nodeName_ == 'code':
            obj_ = code.factory()
            obj_.build(child_)
            self.code.append(obj_)
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'filter':
            obj_ = ValueSet_Filter.factory()
            obj_.build(child_)
            self.filter.append(obj_)
            obj_.original_tagname_ = 'filter'
        super(ValueSet_Include, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet_Include


class ValueSet_Filter(BackboneElement):
    """A value set specifies a set of codes drawn from one or more code
    systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, property=None, op=None, value=None):
        self.original_tagname_ = None
        super(ValueSet_Filter, self).__init__(id, extension, modifierExtension, )
        self.property = property
        self.op = op
        self.value = value
    def factory(*args_, **kwargs_):
        if ValueSet_Filter.subclass:
            return ValueSet_Filter.subclass(*args_, **kwargs_)
        else:
            return ValueSet_Filter(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_property(self): return self.property
    def set_property(self, property): self.property = property
    def get_op(self): return self.op
    def set_op(self, op): self.op = op
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            self.property is not None or
            self.op is not None or
            self.value is not None or
            super(ValueSet_Filter, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet.Filter', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Filter')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet.Filter', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet.Filter'):
        super(ValueSet_Filter, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Filter')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet.Filter', fromsubclass_=False, pretty_print=True):
        super(ValueSet_Filter, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.property is not None:
            self.property.export(outfile, level, namespace_, name_='property', pretty_print=pretty_print)
        if self.op is not None:
            self.op.export(outfile, level, namespace_, name_='op', pretty_print=pretty_print)
        if self.value is not None:
            self.value.export(outfile, level, namespace_, name_='value', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet.Filter', mapping_=None):
        element = super(ValueSet_Filter, self).to_etree(parent_element, name_, mapping_)
        if self.property is not None:
            property_ = self.property
            property_.to_etree(element, name_='property', mapping_=mapping_)
        if self.op is not None:
            op_ = self.op
            op_.to_etree(element, name_='op', mapping_=mapping_)
        if self.value is not None:
            value_ = self.value
            value_.to_etree(element, name_='value', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet_Filter, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'property':
            obj_ = code.factory()
            obj_.build(child_)
            self.property = obj_
            obj_.original_tagname_ = 'property'
        elif nodeName_ == 'op':
            obj_ = FilterOperator.factory()
            obj_.build(child_)
            self.op = obj_
            obj_.original_tagname_ = 'op'
        elif nodeName_ == 'value':
            obj_ = code.factory()
            obj_.build(child_)
            self.value = obj_
            obj_.original_tagname_ = 'value'
        super(ValueSet_Filter, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet_Filter


class ValueSet_Expansion(BackboneElement):
    """A value set specifies a set of codes drawn from one or more code
    systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, identifier=None, timestamp=None, contains=None):
        self.original_tagname_ = None
        super(ValueSet_Expansion, self).__init__(id, extension, modifierExtension, )
        self.identifier = identifier
        self.timestamp = timestamp
        if contains is None:
            self.contains = []
        else:
            self.contains = contains
    def factory(*args_, **kwargs_):
        if ValueSet_Expansion.subclass:
            return ValueSet_Expansion.subclass(*args_, **kwargs_)
        else:
            return ValueSet_Expansion(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_timestamp(self): return self.timestamp
    def set_timestamp(self, timestamp): self.timestamp = timestamp
    def get_contains(self): return self.contains
    def set_contains(self, contains): self.contains = contains
    def add_contains(self, value): self.contains.append(value)
    def insert_contains(self, index, value): self.contains[index] = value
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.timestamp is not None or
            self.contains or
            super(ValueSet_Expansion, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet.Expansion', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Expansion')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet.Expansion', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet.Expansion'):
        super(ValueSet_Expansion, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Expansion')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet.Expansion', fromsubclass_=False, pretty_print=True):
        super(ValueSet_Expansion, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.timestamp is not None:
            self.timestamp.export(outfile, level, namespace_, name_='timestamp', pretty_print=pretty_print)
        for contains_ in self.contains:
            contains_.export(outfile, level, namespace_, name_='contains', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet.Expansion', mapping_=None):
        element = super(ValueSet_Expansion, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.timestamp is not None:
            timestamp_ = self.timestamp
            timestamp_.to_etree(element, name_='timestamp', mapping_=mapping_)
        for contains_ in self.contains:
            contains_.to_etree(element, name_='contains', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet_Expansion, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'timestamp':
            obj_ = instant.factory()
            obj_.build(child_)
            self.timestamp = obj_
            obj_.original_tagname_ = 'timestamp'
        elif nodeName_ == 'contains':
            obj_ = ValueSet_Contains.factory()
            obj_.build(child_)
            self.contains.append(obj_)
            obj_.original_tagname_ = 'contains'
        super(ValueSet_Expansion, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet_Expansion


class ValueSet_Contains(BackboneElement):
    """A value set specifies a set of codes drawn from one or more code
    systems."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, system=None, code=None, display=None, contains=None):
        self.original_tagname_ = None
        super(ValueSet_Contains, self).__init__(id, extension, modifierExtension, )
        self.system = system
        self.code = code
        self.display = display
        if contains is None:
            self.contains = []
        else:
            self.contains = contains
    def factory(*args_, **kwargs_):
        if ValueSet_Contains.subclass:
            return ValueSet_Contains.subclass(*args_, **kwargs_)
        else:
            return ValueSet_Contains(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_system(self): return self.system
    def set_system(self, system): self.system = system
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_display(self): return self.display
    def set_display(self, display): self.display = display
    def get_contains(self): return self.contains
    def set_contains(self, contains): self.contains = contains
    def add_contains(self, value): self.contains.append(value)
    def insert_contains(self, index, value): self.contains[index] = value
    def hasContent_(self):
        if (
            self.system is not None or
            self.code is not None or
            self.display is not None or
            self.contains or
            super(ValueSet_Contains, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ValueSet.Contains', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Contains')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ValueSet.Contains', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ValueSet.Contains'):
        super(ValueSet_Contains, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ValueSet.Contains')
    def exportChildren(self, outfile, level, namespace_='', name_='ValueSet.Contains', fromsubclass_=False, pretty_print=True):
        super(ValueSet_Contains, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.system is not None:
            self.system.export(outfile, level, namespace_, name_='system', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.display is not None:
            self.display.export(outfile, level, namespace_, name_='display', pretty_print=pretty_print)
        for contains_ in self.contains:
            contains_.export(outfile, level, namespace_, name_='contains', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ValueSet.Contains', mapping_=None):
        element = super(ValueSet_Contains, self).to_etree(parent_element, name_, mapping_)
        if self.system is not None:
            system_ = self.system
            system_.to_etree(element, name_='system', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.display is not None:
            display_ = self.display
            display_.to_etree(element, name_='display', mapping_=mapping_)
        for contains_ in self.contains:
            contains_.to_etree(element, name_='contains', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ValueSet_Contains, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'system':
            obj_ = uri.factory()
            obj_.build(child_)
            self.system = obj_
            obj_.original_tagname_ = 'system'
        elif nodeName_ == 'code':
            obj_ = code.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'display':
            obj_ = string.factory()
            obj_.build(child_)
            self.display = obj_
            obj_.original_tagname_ = 'display'
        elif nodeName_ == 'contains':
            obj_ = ValueSet_Contains.factory()
            obj_.build(child_)
            self.contains.append(obj_)
            obj_.original_tagname_ = 'contains'
        super(ValueSet_Contains, self).buildChildren(child_, node, nodeName_, True)
# end class ValueSet_Contains



import unittest
import trytond.tests.test_tryton
from trytond.tests.test_tryton import ModuleTestCase


class HealthTestCase(ModuleTestCase):
    '''
    Test Health module.
    '''
    module = 'health'


def suite():
    suite = trytond.tests.test_tryton.suite()
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
        HealthTestCase))
    return suite



from StringIO import StringIO
from operator import attrgetter
import server.fhir as supermod
from server.common import safe_attrgetter
from .health_mixin import ExportXMLMixin

try:
    from flask import url_for
    RUN_FLASK=True
except:
    from .datastore import dumb_url_generate
    RUN_FLASK=False


class Medication_Map:
    """Holds essential mappings and information for
        the Medication class
    """

    root_search=[]

    resource_search_params={
            '_id': 'token',
            'code': 'token',
            'form': None,
            'container': None,
            'content': None,
            'manufacturer': None,
            'ingredient': None,
            'name': None,
            '_language': None}

    chain_map={}
    search_mapping={
            '_id': ['id'],
            'code': ['name.code'],
            'code:text': ['name.name']
            }

    url_prefixes={}
    model_mapping={'gnuhealth.medicament':
                {
                    'display': 'active_component',
                    'code': 'name.code'
                }}

class health_Medication(supermod.Medication, Medication_Map, ExportXMLMixin):
    def __init__(self, *args, **kwargs):
        rec = kwargs.pop('gnu_record', None)
        super(health_Medication, self).__init__(*args, **kwargs)
        if rec:
            self.set_gnu_medication(rec)

    def set_gnu_medication(self, medication):
        """Set the GNU Health record
        ::::
            params:
                medication ===> Health model
            returns:
                instance

        """
        self.medication = medication
        self.model_type = self.medication.__name__

        # Only certain models
        if self.model_type not in self.model_mapping:
            raise ValueError('Not a valid model')

        self.map = self.model_mapping[self.model_type]

        self.__import_from_gnu_medication()
        self.__set_feed_info()

    def __import_from_gnu_medication(self):
        """Set the data from the model"""
        if self.medication:
            self.set_code(safe_attrgetter(self.medication, self.map['display']),
                            safe_attrgetter(self.medication, self.map['code']))
            self.set_kind('product')
            #self.set_package()
            #self.set_product()
            #self.set_manufacturer()
            #self.set_isBrand()
            #self.set_name()

    def __set_feed_info(self):
        """Set the feed-relevant data"""
        if self.medication:
            if RUN_FLASK:
                uri = url_for('med_record',
                            log_id=self.medication.id,
                            _external=True)
            else:
                uri = dumb_url_generate(['Medication', self.medication.id])
            self.feed={'id': uri,
                        'published': self.medication.create_date,
                        'updated': self.medication.write_date or self.medication.create_date,
                        'title': self.medication.name.name
                        }

    def set_name(self, name):
        """Set medication name"""

        #TODO Need common/commercial names
        if name:
            n = supermod.string(value=str(name))
            super(health_Medication, self).set_name(n)

    def set_code(self, name, code):
        """Extends superclass for convenience

        Keyword arguments:
        code -- code value
        name -- name of the medication
        """

        #TODO Better info, use recognized codes
        if name:
            c = supermod.Coding()
            c.display = supermod.string(value=str(name))
            if code:
                c.code = supermod.code(value=code)
            cc = supermod.CodeableConcept()
            cc.coding=[c]
            super(health_Medication, self).set_code(cc)

    def set_kind(self, kind='product'):
        """Extends superclass for convenience

        Keyword arguments:
        kind - basically, product or package
        """

        if kind in ['product', 'package']:
            c = supermod.code(value=kind)
            super(health_Medication, self).set_kind(c)

supermod.Medication.subclass=health_Medication




STATUS_CODES={
        100:    "Continue",
        102:    "Processing",
        200:    "Ok",
        201:    "Created",
        204:    "No Content",
        207:    "Multi-Status",
        400:    "Bad Request",
        403:    "Forbidden",
        404:    "Not Found",
        405:    "Method Not Allowed",
        409:    "Conflict",
        412:    "Precondition failed",
        422:    "Unprocessable Entity",
        423:    "Locked",
        424:    "Failed Dependency",
        502:    "Bad Gateway",
        507:    "Insufficient Storage"
}



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateTransition, StateView, Button
from trytond.transaction import Transaction
from trytond.pool import Pool


__all__ = ['CreateLabTestOrderInit', 'CreateLabTestOrder', 'RequestTest',
    'RequestPatientLabTestStart', 'RequestPatientLabTest']


class CreateLabTestOrderInit(ModelView):
    'Create Test Report Init'
    __name__ = 'gnuhealth.lab.test.create.init'


class CreateLabTestOrder(Wizard):
    'Create Lab Test Report'
    __name__ = 'gnuhealth.lab.test.create'

    start = StateView('gnuhealth.lab.test.create.init',
        'health_lab.view_lab_make_test', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Create Test Order', 'create_lab_test', 'tryton-ok', True),
            ])

    create_lab_test = StateTransition()

    def transition_create_lab_test(self):
        TestRequest = Pool().get('gnuhealth.patient.lab.test')
        Lab = Pool().get('gnuhealth.lab')

        tests_report_data = []

        tests = TestRequest.browse(Transaction().context.get('active_ids'))

        for lab_test_order in tests:

            test_cases = []
            test_report_data = {}

            if lab_test_order.state == 'ordered':
                self.raise_user_error(
                    "The Lab test order is already created")

            test_report_data['test'] = lab_test_order.name.id
            test_report_data['patient'] = lab_test_order.patient_id.id
            if lab_test_order.doctor_id:
                test_report_data['requestor'] = lab_test_order.doctor_id.id
            test_report_data['date_requested'] = lab_test_order.date
            test_report_data['request_order'] = lab_test_order.request

            for critearea in lab_test_order.name.critearea:
                test_cases.append(('create', [{
                        'name': critearea.name,
                        'sequence': critearea.sequence,
                        'lower_limit': critearea.lower_limit,
                        'upper_limit': critearea.upper_limit,
                        'normal_range': critearea.normal_range,
                        'units': critearea.units and critearea.units.id,
                    }]))
            test_report_data['critearea'] = test_cases

            tests_report_data.append(test_report_data)

        Lab.create(tests_report_data)
        TestRequest.write(tests, {'state': 'ordered'})

        return 'end'


class RequestTest(ModelView):
    'Request - Test'
    __name__ = 'gnuhealth.request-test'
    _table = 'gnuhealth_request_test'

    request = fields.Many2One('gnuhealth.patient.lab.test.request.start',
        'Request', required=True)
    test = fields.Many2One('gnuhealth.lab.test_type', 'Test', required=True)


class RequestPatientLabTestStart(ModelView):
    'Request Patient Lab Test Start'
    __name__ = 'gnuhealth.patient.lab.test.request.start'

    date = fields.DateTime('Date')
    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)
    doctor = fields.Many2One('gnuhealth.healthprofessional', 'Doctor',
        help="Doctor who Request the lab tests.")
    tests = fields.Many2Many('gnuhealth.request-test', 'request', 'test',
        'Tests', required=True)
    urgent = fields.Boolean('Urgent')

    @staticmethod
    def default_date():
        return datetime.now()

    @staticmethod
    def default_patient():
        if Transaction().context.get('active_model') == 'gnuhealth.patient':
            return Transaction().context.get('active_id')

    @staticmethod
    def default_doctor():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        hp = HealthProf.get_health_professional()
        if not hp:
            RequestPatientLabTestStart.raise_user_error(
                "No health professional associated to this user !")
        return hp

class RequestPatientLabTest(Wizard):
    'Request Patient Lab Test'
    __name__ = 'gnuhealth.patient.lab.test.request'

    start = StateView('gnuhealth.patient.lab.test.request.start',
        'health_lab.patient_lab_test_request_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Request', 'request', 'tryton-ok', default=True),
            ])
    request = StateTransition()

    def transition_request(self):
        PatientLabTest = Pool().get('gnuhealth.patient.lab.test')
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        config = Config(1)
        request_number = Sequence.get_id(config.lab_request_sequence.id)
        lab_tests = []
        for test in self.start.tests:
            lab_test = {}
            lab_test['request'] = request_number
            lab_test['name'] = test.id
            lab_test['patient_id'] = self.start.patient.id
            if self.start.doctor:
                lab_test['doctor_id'] = self.start.doctor.id
            lab_test['date'] = self.start.date
            lab_test['urgent'] = self.start.urgent
            lab_tests.append(lab_test)
        PatientLabTest.create(lab_tests)

        return 'end'



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
from trytond.pool import Pool
from .webdav import *


def register():
    Pool.register(
        Collection,
        Share,
        Attachment,
        module='webdav', type_='model')



class immunizationSite:
    contents=[
        { 'code': 'LVL', 'display': 'left vastus lateralis' },
        { 'code': 'RVL', 'display': 'right vastus lateralis' },
        { 'code': 'LD', 'display': 'left deltoid' },
        { 'code': 'RD', 'display': 'right deltoid' },
        { 'code': 'LALT', 'display': 'left anterolateral fat of thigh' },
        { 'code': 'RALT', 'display': 'right anterolateral fat of thigh' },
        { 'code': 'LPUA', 'display': 'left posterolateral fat of upper arm' },
        { 'code': 'RPUA', 'display': 'right posterolateral fat of upper arm' },
        { 'code': 'LFA', 'display': 'left fore arm' },
        { 'code': 'RFA', 'display': 'right fore arm' }]

__all__=['immunizationSite']



from StringIO import StringIO
from datetime import datetime
import server.fhir as supermod
from .health_mixin import ExportXMLMixin

# Sub-class OperationOutcome

class health_OperationOutcome(supermod.OperationOutcome, ExportXMLMixin):
    '''This provides an easy sub-class to operation outcomes'''

    def add_issue(self, severity=None, location=None, details=None):
        issue=health_OperationOutcome_Issue()
        issue.set_severity(severity)
        issue.set_details(details)
        issue.add_location(location)
        super(health_OperationOutcome, self).add_issue(issue)

supermod.OperationOutcome.subclass=health_OperationOutcome

class health_OperationOutcome_Issue(supermod.OperationOutcome_Issue):
    '''This provides an easy sub-class to operation outcome issues'''
    def set_details(self, details):
        if details:
            st = supermod.string(value=details)
            super(health_OperationOutcome_Issue, self).set_details(st)

    def add_location(self, location):
        if location:
            st = supermod.string(value=location)
            super(health_OperationOutcome_Issue, self).add_location(st)

    def set_severity(self, severity):
        if severity:
            sev=supermod.misc.IssueSeverity(value=severity)
            super(health_OperationOutcome_Issue, self).set_severity(sev)

supermod.OperationOutcome_Issue.subclass=health_OperationOutcome_Issue



from .base_classes import *
from .support_functions import *


class Device(Resource):
    """This resource identifies an instance of a manufactured thing that is
    used in the provision of healthcare without being substantially
    changed through that activity. The device may be a machine, an
    insert, a computer, an application, etc. This includes durable
    (reusable) medical equipment as well as disposable equipment
    used for diagnostic, treatment, and research for healthcare and
    public health.If the element is present, it must have either a
    @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, type_=None, manufacturer=None, model=None, version=None, expiry=None, udi=None, lotNumber=None, owner=None, location=None, patient=None, contact=None, url=None):
        self.original_tagname_ = None
        super(Device, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.type_ = type_
        self.manufacturer = manufacturer
        self.model = model
        self.version = version
        self.expiry = expiry
        self.udi = udi
        self.lotNumber = lotNumber
        self.owner = owner
        self.location = location
        self.patient = patient
        if contact is None:
            self.contact = []
        else:
            self.contact = contact
        self.url = url
    def factory(*args_, **kwargs_):
        if Device.subclass:
            return Device.subclass(*args_, **kwargs_)
        else:
            return Device(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_manufacturer(self): return self.manufacturer
    def set_manufacturer(self, manufacturer): self.manufacturer = manufacturer
    def get_model(self): return self.model
    def set_model(self, model): self.model = model
    def get_version(self): return self.version
    def set_version(self, version): self.version = version
    def get_expiry(self): return self.expiry
    def set_expiry(self, expiry): self.expiry = expiry
    def get_udi(self): return self.udi
    def set_udi(self, udi): self.udi = udi
    def get_lotNumber(self): return self.lotNumber
    def set_lotNumber(self, lotNumber): self.lotNumber = lotNumber
    def get_owner(self): return self.owner
    def set_owner(self, owner): self.owner = owner
    def get_location(self): return self.location
    def set_location(self, location): self.location = location
    def get_patient(self): return self.patient
    def set_patient(self, patient): self.patient = patient
    def get_contact(self): return self.contact
    def set_contact(self, contact): self.contact = contact
    def add_contact(self, value): self.contact.append(value)
    def insert_contact(self, index, value): self.contact[index] = value
    def get_url(self): return self.url
    def set_url(self, url): self.url = url
    def hasContent_(self):
        if (
            self.identifier or
            self.type_ is not None or
            self.manufacturer is not None or
            self.model is not None or
            self.version is not None or
            self.expiry is not None or
            self.udi is not None or
            self.lotNumber is not None or
            self.owner is not None or
            self.location is not None or
            self.patient is not None or
            self.contact or
            self.url is not None or
            super(Device, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Device', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Device')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Device', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Device'):
        super(Device, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Device')
    def exportChildren(self, outfile, level, namespace_='', name_='Device', fromsubclass_=False, pretty_print=True):
        super(Device, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.manufacturer is not None:
            self.manufacturer.export(outfile, level, namespace_, name_='manufacturer', pretty_print=pretty_print)
        if self.model is not None:
            self.model.export(outfile, level, namespace_, name_='model', pretty_print=pretty_print)
        if self.version is not None:
            self.version.export(outfile, level, namespace_, name_='version', pretty_print=pretty_print)
        if self.expiry is not None:
            self.expiry.export(outfile, level, namespace_, name_='expiry', pretty_print=pretty_print)
        if self.udi is not None:
            self.udi.export(outfile, level, namespace_, name_='udi', pretty_print=pretty_print)
        if self.lotNumber is not None:
            self.lotNumber.export(outfile, level, namespace_, name_='lotNumber', pretty_print=pretty_print)
        if self.owner is not None:
            self.owner.export(outfile, level, namespace_, name_='owner', pretty_print=pretty_print)
        if self.location is not None:
            self.location.export(outfile, level, namespace_, name_='location', pretty_print=pretty_print)
        if self.patient is not None:
            self.patient.export(outfile, level, namespace_, name_='patient', pretty_print=pretty_print)
        for contact_ in self.contact:
            contact_.export(outfile, level, namespace_, name_='contact', pretty_print=pretty_print)
        if self.url is not None:
            self.url.export(outfile, level, namespace_, name_='url', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Device', mapping_=None):
        element = super(Device, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.manufacturer is not None:
            manufacturer_ = self.manufacturer
            manufacturer_.to_etree(element, name_='manufacturer', mapping_=mapping_)
        if self.model is not None:
            model_ = self.model
            model_.to_etree(element, name_='model', mapping_=mapping_)
        if self.version is not None:
            version_ = self.version
            version_.to_etree(element, name_='version', mapping_=mapping_)
        if self.expiry is not None:
            expiry_ = self.expiry
            expiry_.to_etree(element, name_='expiry', mapping_=mapping_)
        if self.udi is not None:
            udi_ = self.udi
            udi_.to_etree(element, name_='udi', mapping_=mapping_)
        if self.lotNumber is not None:
            lotNumber_ = self.lotNumber
            lotNumber_.to_etree(element, name_='lotNumber', mapping_=mapping_)
        if self.owner is not None:
            owner_ = self.owner
            owner_.to_etree(element, name_='owner', mapping_=mapping_)
        if self.location is not None:
            location_ = self.location
            location_.to_etree(element, name_='location', mapping_=mapping_)
        if self.patient is not None:
            patient_ = self.patient
            patient_.to_etree(element, name_='patient', mapping_=mapping_)
        for contact_ in self.contact:
            contact_.to_etree(element, name_='contact', mapping_=mapping_)
        if self.url is not None:
            url_ = self.url
            url_.to_etree(element, name_='url', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Device, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'manufacturer':
            obj_ = string.factory()
            obj_.build(child_)
            self.manufacturer = obj_
            obj_.original_tagname_ = 'manufacturer'
        elif nodeName_ == 'model':
            obj_ = string.factory()
            obj_.build(child_)
            self.model = obj_
            obj_.original_tagname_ = 'model'
        elif nodeName_ == 'version':
            obj_ = string.factory()
            obj_.build(child_)
            self.version = obj_
            obj_.original_tagname_ = 'version'
        elif nodeName_ == 'expiry':
            obj_ = date.factory()
            obj_.build(child_)
            self.expiry = obj_
            obj_.original_tagname_ = 'expiry'
        elif nodeName_ == 'udi':
            obj_ = string.factory()
            obj_.build(child_)
            self.udi = obj_
            obj_.original_tagname_ = 'udi'
        elif nodeName_ == 'lotNumber':
            obj_ = string.factory()
            obj_.build(child_)
            self.lotNumber = obj_
            obj_.original_tagname_ = 'lotNumber'
        elif nodeName_ == 'owner':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.owner = obj_
            obj_.original_tagname_ = 'owner'
        elif nodeName_ == 'location':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.location = obj_
            obj_.original_tagname_ = 'location'
        elif nodeName_ == 'patient':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.patient = obj_
            obj_.original_tagname_ = 'patient'
        elif nodeName_ == 'contact':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.contact.append(obj_)
            obj_.original_tagname_ = 'contact'
        elif nodeName_ == 'url':
            obj_ = uri.factory()
            obj_.build(child_)
            self.url = obj_
            obj_.original_tagname_ = 'url'
        super(Device, self).buildChildren(child_, node, nodeName_, True)
# end class Device
